@@ -2,7 +2,7 @@
 # frozen_string_literal: true
 
 require 'pg' unless defined?( PG )
-require 'io/wait' unless ::IO.public_instance_methods(false).include?(:wait_readable)
+require 'io/wait' unless ::IO.public_instance_methods(false).include?(:wait_readable) # for ruby < 3.0
 require 'socket'
 
 # The PostgreSQL connection class. The interface for this class is based on
@@ -117,7 +117,7 @@ class PG::Connection
 		return str
 	end
 
-	BinarySignature = "PGCOPY\n\377\r\n\0".b
+	BinarySignature = "PGCOPY\n\377\r\n\0"
 	private_constant :BinarySignature
 
 	#  call-seq:
@@ -166,7 +166,10 @@ class PG::Connection
 	#     conn.put_copy_data ['more', 'data', 'to', 'copy']
 	#   end
 	#
-	# Also PG::BinaryEncoder::CopyRow can be used to send data in binary format to the server.
+	# All 4 CopyRow classes can take a type map to specify how the columns are mapped to and from the database format.
+	# For details see the particular CopyRow class description.
+	#
+	# PG::BinaryEncoder::CopyRow can be used to send data in binary format to the server.
 	# In this case copy_data generates the header and trailer data automatically:
 	#   enco = PG::BinaryEncoder::CopyRow.new
 	#   conn.copy_data "COPY my_table FROM STDIN (FORMAT binary)", enco do
@@ -306,6 +309,11 @@ class PG::Connection
 		rollback = false
 		exec "BEGIN"
 		yield(self)
+	rescue PG::RollbackTransaction
+		rollback = true
+		cancel if transaction_status == PG::PQTRANS_ACTIVE
+		block
+		exec "ROLLBACK"
 	rescue Exception
 		rollback = true
 		cancel if transaction_status == PG::PQTRANS_ACTIVE
@@ -348,21 +356,18 @@ class PG::Connection
 		end
 	end
 
-	# Method 'ssl_attribute' was introduced in PostgreSQL 9.5.
-	if self.instance_methods.find{|m| m.to_sym == :ssl_attribute }
-		# call-seq:
-		#   conn.ssl_attributes -> Hash<String,String>
-		#
-		# Returns SSL-related information about the connection as key/value pairs
-		#
-		# The available attributes varies depending on the SSL library being used,
-		# and the type of connection.
-		#
-		# See also #ssl_attribute
-		def ssl_attributes
-			ssl_attribute_names.each.with_object({}) do |n,h|
-				h[n] = ssl_attribute(n)
-			end
+	# call-seq:
+	#   conn.ssl_attributes -> Hash<String,String>
+	#
+	# Returns SSL-related information about the connection as key/value pairs
+	#
+	# The available attributes varies depending on the SSL library being used,
+	# and the type of connection.
+	#
+	# See also #ssl_attribute
+	def ssl_attributes
+		ssl_attribute_names.each.with_object({}) do |n,h|
+			h[n] = ssl_attribute(n)
 		end
 	end
 
@@ -493,7 +498,7 @@ class PG::Connection
 	# See also #copy_data.
 	#
 	def put_copy_data(buffer, encoder=nil)
-		# sync_put_copy_data does a non-blocking attept to flush data.
+		# sync_put_copy_data does a non-blocking attempt to flush data.
 		until res=sync_put_copy_data(buffer, encoder)
 			# It didn't flush immediately and allocation of more buffering memory failed.
 			# Wait for all data sent by doing a blocking flush.
@@ -531,6 +536,25 @@ class PG::Connection
 	end
 	alias async_put_copy_end put_copy_end
 
+	if method_defined? :send_pipeline_sync
+		# call-seq:
+		#    conn.pipeline_sync
+		#
+		# Marks a synchronization point in a pipeline by sending a sync message and flushing the send buffer.
+		# This serves as the delimiter of an implicit transaction and an error recovery point.
+		#
+		# See enter_pipeline_mode
+		#
+		# Raises PG::Error if the connection is not in pipeline mode or sending a sync message failed.
+		#
+		# Available since PostgreSQL-14
+		def pipeline_sync(*args)
+			send_pipeline_sync(*args)
+			flush
+		end
+		alias async_pipeline_sync pipeline_sync
+	end
+
 	if method_defined? :sync_encrypt_password
 		# call-seq:
 		#    conn.encrypt_password( password, username, algorithm=nil ) -> String
@@ -565,7 +589,9 @@ class PG::Connection
 	# Resets the backend connection. This method closes the
 	# backend connection and tries to re-connect.
 	def reset
-		iopts = conninfo_hash.compact
+		# Use connection options from PG::Connection.new to reconnect with the same options but with renewed DNS resolution.
+		# Use conninfo_hash as a fallback when connect_start was used to create the connection object.
+		iopts = @iopts_for_reset || conninfo_hash.compact
 		if iopts[:host] && !iopts[:host].empty? && PG.library_version >= 100000
 			iopts = self.class.send(:resolve_hosts, iopts)
 		end
@@ -576,130 +602,197 @@ class PG::Connection
 	end
 	alias async_reset reset
 
-	# call-seq:
-	#    conn.cancel() -> String
-	#
-	# Requests cancellation of the command currently being
-	# processed.
-	#
-	# Returns +nil+ on success, or a string containing the
-	# error message if a failure occurs.
-	def cancel
-		be_pid = backend_pid
-		be_key = backend_key
-		cancel_request = [0x10, 1234, 5678, be_pid, be_key].pack("NnnNN")
+	if defined?(PG::CancelConnection)
+		# PostgreSQL-17+
 
-		if Fiber.respond_to?(:scheduler) && Fiber.scheduler && RUBY_PLATFORM =~ /mingw|mswin/
-			# Ruby's nonblocking IO is not really supported on Windows.
-			# We work around by using threads and explicit calls to wait_readable/wait_writable.
-			cl = Thread.new(socket_io.remote_address) { |ra| ra.connect }.value
-			begin
-				cl.write_nonblock(cancel_request)
-			rescue IO::WaitReadable, Errno::EINTR
-				cl.wait_writable
-				retry
-			end
-			begin
-				cl.read_nonblock(1)
-			rescue IO::WaitReadable, Errno::EINTR
-				cl.wait_readable
-				retry
-			rescue EOFError
-			end
-		elsif RUBY_ENGINE == 'truffleruby'
-			begin
-				cl = socket_io.remote_address.connect
-			rescue NotImplementedError
-				# Workaround for truffleruby < 21.3.0
-				cl2 = Socket.for_fd(socket_io.fileno)
-				cl2.autoclose = false
-				adr = cl2.remote_address
-				if adr.ip?
-					cl = TCPSocket.new(adr.ip_address, adr.ip_port)
-					cl.autoclose = false
-				else
-					cl = UNIXSocket.new(adr.unix_path)
-					cl.autoclose = false
-				end
-			end
-			cl.write(cancel_request)
-			cl.read(1)
-		else
-			cl = socket_io.remote_address.connect
-			# Send CANCEL_REQUEST_CODE and parameters
-			cl.write(cancel_request)
-			# Wait for the postmaster to close the connection, which indicates that it's processed the request.
-			cl.read(1)
+		def sync_cancel
+			cancon = PG::CancelConnection.new(self)
+			cancon.sync_cancel
+		rescue PG::Error => err
+			err.to_s
 		end
 
-		cl.close
-		nil
-	rescue SystemCallError => err
-		err.to_s
+		# call-seq:
+		#    conn.cancel() -> String
+		#
+		# Requests cancellation of the command currently being
+		# processed.
+		#
+		# Returns +nil+ on success, or a string containing the
+		# error message if a failure occurs.
+		#
+		# On PostgreSQL-17+ client libaray the class PG::CancelConnection is used.
+		# On older client library a pure ruby implementation is used.
+		def cancel
+			cancon = PG::CancelConnection.new(self)
+			cancon.async_cancel
+		rescue PG::Error => err
+			err.to_s
+		end
+
+	else
+
+		# PostgreSQL < 17
+
+		def cancel
+			be_pid = backend_pid
+			be_key = backend_key
+			cancel_request = [0x10, 1234, 5678, be_pid, be_key].pack("NnnNN")
+
+			if Fiber.respond_to?(:scheduler) && Fiber.scheduler && RUBY_PLATFORM =~ /mingw|mswin/
+				# Ruby's nonblocking IO is not really supported on Windows.
+				# We work around by using threads and explicit calls to wait_readable/wait_writable.
+				cl = Thread.new(socket_io.remote_address) { |ra| ra.connect }.value
+				begin
+					cl.write_nonblock(cancel_request)
+				rescue IO::WaitReadable, Errno::EINTR
+					cl.wait_writable
+					retry
+				end
+				begin
+					cl.read_nonblock(1)
+				rescue IO::WaitReadable, Errno::EINTR
+					cl.wait_readable
+					retry
+				rescue EOFError
+				end
+			else
+				cl = socket_io.remote_address.connect
+				# Send CANCEL_REQUEST_CODE and parameters
+				cl.write(cancel_request)
+				# Wait for the postmaster to close the connection, which indicates that it's processed the request.
+				cl.read(1)
+			end
+
+			cl.close
+			nil
+		rescue SystemCallError => err
+			err.to_s
+		end
 	end
 	alias async_cancel cancel
 
+	module Pollable
+		# Track the progress of the connection, waiting for the socket to become readable/writable before polling it.
+		#
+		# Connecting to multiple hosts is done like so:
+		# - All hosts are passed to PG::Connection.connect_start
+		# - As soon as the host is tried to connect the related host is removed from the hosts list
+		# - When the polling status changes to `PG::PGRES_POLLING_OK` the connection is returned and ready to use.
+		# - When the polling status changes to `PG::PGRES_POLLING_FAILED` connecting is aborted and a PG::ConnectionBad is raised with details to all connection attepts.
+		# - When a timeout occurs, connecting is restarted with the remaining hosts.
+		#
+		# The downside is that this connects only once to hosts which are listed twice when they timeout.
+		private def polling_loop(poll_meth)
+			connect_timeout = conninfo_hash[:connect_timeout]
+			if (timeo = connect_timeout.to_i) && timeo > 0
+				host_count = (conninfo_hash[:hostaddr].to_s.empty? ? conninfo_hash[:host] : conninfo_hash[:hostaddr]).to_s.count(",") + 1
+				stop_time = timeo * host_count + Process.clock_gettime(Process::CLOCK_MONOTONIC)
+			end
+			iopts = conninfo_hash.compact
+			connection_errors = []
+			poll_status = PG::PGRES_POLLING_WRITING
+
+			until poll_status == PG::PGRES_POLLING_OK ||
+					poll_status == PG::PGRES_POLLING_FAILED
+
+				# Set single timeout to parameter "connect_timeout" but
+				# don't exceed total connection time of number-of-hosts * connect_timeout.
+				timeout = [timeo, stop_time - Process.clock_gettime(Process::CLOCK_MONOTONIC)].min if stop_time
+
+				hostcnt = remove_current_host(iopts)
+
+				event = if !timeout || timeout >= 0
+					# If the socket needs to read, wait 'til it becomes readable to poll again
+					case poll_status
+					when PG::PGRES_POLLING_READING
+						if defined?(IO::READABLE) # ruby-3.0+
+							socket_io.wait(IO::READABLE | IO::PRIORITY, timeout)
+						else
+							IO.select([socket_io], nil, [socket_io], timeout)
+						end
+
+					# ...and the same for when the socket needs to write
+					when PG::PGRES_POLLING_WRITING
+						if defined?(IO::WRITABLE) # ruby-3.0+
+							# Use wait instead of wait_readable, since connection errors are delivered as
+							# exceptional/priority events on Windows.
+							socket_io.wait(IO::WRITABLE | IO::PRIORITY, timeout)
+						else
+							# io#wait on ruby-2.x doesn't wait for priority, so fallback to IO.select
+							IO.select(nil, [socket_io], [socket_io], timeout)
+						end
+					end
+				end
+
+				# connection to server at "localhost" (127.0.0.1), port 5433 failed: timeout expired (PG::ConnectionBad)
+				# connection to server on socket "/var/run/postgresql/.s.PGSQL.5433" failed: No such file or directory
+				unless event
+					connection_errors << (error_message + "timeout expired")
+					if hostcnt > 0
+						reset_start2(self.class.parse_connect_args(iopts))
+						# Restart polling with waiting for writable.
+						# Otherwise "not connected" error is raised on Windows.
+						poll_status = PG::PGRES_POLLING_WRITING
+						next
+					else
+						finish
+						raise PG::ConnectionBad.new(connection_errors.join("\n").b, connection: self)
+					end
+				end
+
+				# Check to see if it's finished or failed yet
+				poll_status = send( poll_meth )
+			end
+
+			unless status == PG::CONNECTION_OK
+				msg = error_message
+				finish
+				raise PG::ConnectionBad.new(connection_errors.map{|e| e + "\n" }.join.b + msg, connection: self)
+			end
+		end
+
+		# Remove the host to which the connection is currently established from the option hash.
+		# Affected options are:
+		# - :host
+		# - :hostaddr
+		# - :port
+		#
+		# Return the number of remaining hosts.
+		private def remove_current_host(iopts)
+			ihosts = iopts[:host]&.split(",", -1)
+			ihostaddrs = iopts[:hostaddr]&.split(",", -1)
+			iports = iopts[:port]&.split(",", -1)
+			iports = iports * (ihosts || ihostaddrs || [1]).size if iports&.size == 1
+
+			idx = (ihosts || ihostaddrs || iports).index.with_index do |_, i|
+				(ihosts ? ihosts[i] == host : true) &&
+				(ihostaddrs && respond_to?(:hostaddr, true) ? ihostaddrs[i] == hostaddr : true) &&
+				(iports ? iports[i].to_i == port : true)
+			end
+
+			if idx
+				ihosts&.delete_at(idx)
+				ihostaddrs&.delete_at(idx)
+				iports&.delete_at(idx)
+
+				iopts.merge!(
+					host: ihosts.join(",")) if ihosts
+				iopts.merge!(
+					hostaddr: ihostaddrs.join(",")) if ihostaddrs
+				iopts.merge!(
+					port: iports.join(",")) if iports
+			end
+
+			(ihosts || ihostaddrs || iports).size
+		end
+	end
+
+	include Pollable
+
 	private def async_connect_or_reset(poll_meth)
 		# Track the progress of the connection, waiting for the socket to become readable/writable before polling it
-
-		if (timeo = conninfo_hash[:connect_timeout].to_i) && timeo > 0
-			# Lowest timeout is 2 seconds - like in libpq
-			timeo = [timeo, 2].max
-			host_count = conninfo_hash[:host].to_s.count(",") + 1
-			stop_time = timeo * host_count + Process.clock_gettime(Process::CLOCK_MONOTONIC)
-		end
-
-		poll_status = PG::PGRES_POLLING_WRITING
-		until poll_status == PG::PGRES_POLLING_OK ||
-				poll_status == PG::PGRES_POLLING_FAILED
-
-			# Set single timeout to parameter "connect_timeout" but
-			# don't exceed total connection time of number-of-hosts * connect_timeout.
-			timeout = [timeo, stop_time - Process.clock_gettime(Process::CLOCK_MONOTONIC)].min if stop_time
-			event = if !timeout || timeout >= 0
-				# If the socket needs to read, wait 'til it becomes readable to poll again
-				case poll_status
-				when PG::PGRES_POLLING_READING
-					if defined?(IO::READABLE) # ruby-3.0+
-						socket_io.wait(IO::READABLE | IO::PRIORITY, timeout)
-					else
-						IO.select([socket_io], nil, [socket_io], timeout)
-					end
-
-				# ...and the same for when the socket needs to write
-				when PG::PGRES_POLLING_WRITING
-					if defined?(IO::WRITABLE) # ruby-3.0+
-						# Use wait instead of wait_readable, since connection errors are delivered as
-						# exceptional/priority events on Windows.
-						socket_io.wait(IO::WRITABLE | IO::PRIORITY, timeout)
-					else
-						# io#wait on ruby-2.x doesn't wait for priority, so fallback to IO.select
-						IO.select(nil, [socket_io], [socket_io], timeout)
-					end
-				end
-			end
-			# connection to server at "localhost" (127.0.0.1), port 5433 failed: timeout expired (PG::ConnectionBad)
-			# connection to server on socket "/var/run/postgresql/.s.PGSQL.5433" failed: No such file or directory
-			unless event
-				if self.class.send(:host_is_named_pipe?, host)
-					connhost = "on socket \"#{host}\""
-				elsif respond_to?(:hostaddr)
-					connhost = "at \"#{host}\" (#{hostaddr}), port #{port}"
-				else
-					connhost = "at \"#{host}\", port #{port}"
-				end
-				raise PG::ConnectionBad.new("connection to server #{connhost} failed: timeout expired", connection: self)
-			end
-
-			# Check to see if it's finished or failed yet
-			poll_status = send( poll_meth )
-		end
-
-		unless status == PG::CONNECTION_OK
-			msg = error_message
-			finish
-			raise PG::ConnectionBad.new(msg, connection: self)
-		end
+		polling_loop(poll_meth)
 
 		# Set connection to nonblocking to handle all blocking states in ruby.
 		# That way a fiber scheduler is able to handle IO requests.
@@ -715,7 +808,7 @@ class PG::Connection
 		#    PG::Connection.new(connection_string) -> conn
 		#    PG::Connection.new(host, port, options, tty, dbname, user, password) ->  conn
 		#
-		# Create a connection to the specified server.
+		# === Create a connection to the specified server.
 		#
 		# +connection_hash+ must be a ruby Hash with connection parameters.
 		# See the {list of valid parameters}[https://www.postgresql.org/docs/current/libpq-connect.html#LIBPQ-PARAMKEYWORDS] in the PostgreSQL documentation.
@@ -739,7 +832,13 @@ class PG::Connection
 		# [+password+]
 		#   login password
 		#
-		# Examples:
+		#
+		# If the Ruby default internal encoding is set (i.e., <code>Encoding.default_internal != nil</code>), the
+		# connection will have its +client_encoding+ set accordingly.
+		#
+		# Raises a PG::Error if the connection fails.
+		#
+		# === Examples:
 		#
 		#   # Connect using all defaults
 		#   PG::Connection.new
@@ -756,10 +855,18 @@ class PG::Connection
 		#   # As an URI
 		#   PG::Connection.new( "postgresql://user:pass@pgsql.example.com:5432/testdb?sslmode=require" )
 		#
-		# If the Ruby default internal encoding is set (i.e., <code>Encoding.default_internal != nil</code>), the
-		# connection will have its +client_encoding+ set accordingly.
+		# === Specifying Multiple Hosts
+		#
+		# It is possible to specify multiple hosts to connect to, so that they are tried in the given order or optionally in random order.
+		# In the Keyword/Value format, the host, hostaddr, and port options accept comma-separated lists of values.
+		# The {details to libpq}[https://www.postgresql.org/docs/current/libpq-connect.html#LIBPQ-MULTIPLE-HOSTS] describe how it works, but there are two small differences how ruby-pg handles multiple hosts:
+		# - All hosts are resolved before the first connection is tried.
+		#   This means that when +load_balance_hosts+ is set to +random+, then all resolved addresses are tried randomly in one level.
+		#   When a host resolves to more than one address, it is therefore tried more often than a host that has only one address.
+		# - When a timeout occurs due to the value of +connect_timeout+, then the given +host+, +hostaddr+ and +port+ combination is not tried a second time, even if it's specified several times.
+		#   It's still possible to do load balancing with +load_balance_hosts+ set to +random+ and to increase the number of connections a node gets, when the hostname is provided multiple times in the host string.
+		#   This is because in non-timeout cases the host is tried multiple times.
 		#
-		# Raises a PG::Error if the connection fails.
 		def new(*args)
 			conn = connect_to_hosts(*args)
 
@@ -817,6 +924,15 @@ class PG::Connection
 			iopts = PG::Connection.conninfo_parse(option_string).each_with_object({}){|h, o| o[h[:keyword].to_sym] = h[:val] if h[:val] }
 			iopts = PG::Connection.conndefaults.each_with_object({}){|h, o| o[h[:keyword].to_sym] = h[:val] if h[:val] }.merge(iopts)
 
+			if PG::BUNDLED_LIBPQ_WITH_UNIXSOCKET && iopts[:host].to_s.empty? && iopts[:hostaddr].to_s.empty?
+				# Many distors patch the hardcoded default UnixSocket path in libpq to /var/run/postgresql instead of /tmp .
+				# We simply try them all.
+				iopts[:host] = "/var/run/postgresql" + # Ubuntu, Debian, Fedora, Opensuse
+					",/run/postgresql" + # Alpine, Archlinux, Gentoo
+					",/tmp" # Stock PostgreSQL
+			end
+
+			iopts_for_reset = iopts
 			if iopts[:hostaddr]
 				# hostaddr is provided -> no need to resolve hostnames
 
@@ -830,6 +946,8 @@ class PG::Connection
 
 			raise PG::ConnectionBad, conn.error_message if conn.status == PG::CONNECTION_BAD
 
+			# save the connection options for conn.reset
+			conn.instance_variable_set(:@iopts_for_reset, iopts_for_reset)
 			conn.send(:async_connect_or_reset, :connect_poll)
 			conn
 		end
@@ -886,14 +1004,29 @@ class PG::Connection
 		private_constant :REDIRECT_CLASS_METHODS
 
 		# These methods are affected by PQsetnonblocking
-		REDIRECT_SEND_METHODS = PG.make_shareable({
+		REDIRECT_SEND_METHODS = {
 			:isnonblocking => [:async_isnonblocking, :sync_isnonblocking],
 			:nonblocking? => [:async_isnonblocking, :sync_isnonblocking],
 			:put_copy_data => [:async_put_copy_data, :sync_put_copy_data],
 			:put_copy_end => [:async_put_copy_end, :sync_put_copy_end],
 			:flush => [:async_flush, :sync_flush],
-		})
+		}
 		private_constant :REDIRECT_SEND_METHODS
+		if PG::Connection.instance_methods.include? :sync_pipeline_sync
+			if PG::Connection.instance_methods.include? :send_pipeline_sync
+				# PostgreSQL-17+
+				REDIRECT_SEND_METHODS.merge!({
+					:pipeline_sync => [:async_pipeline_sync, :sync_pipeline_sync],
+				})
+			else
+				# PostgreSQL-14+
+				REDIRECT_SEND_METHODS.merge!({
+					:pipeline_sync => [:sync_pipeline_sync, :sync_pipeline_sync],
+				})
+			end
+		end
+		PG.make_shareable(REDIRECT_SEND_METHODS)
+
 		REDIRECT_METHODS = {
 			:exec => [:async_exec, :sync_exec],
 			:query => [:async_exec, :sync_exec],
@@ -910,12 +1043,13 @@ class PG::Connection
 			:set_client_encoding => [:async_set_client_encoding, :sync_set_client_encoding],
 			:client_encoding= => [:async_set_client_encoding, :sync_set_client_encoding],
 			:cancel => [:async_cancel, :sync_cancel],
+			:encrypt_password => [:async_encrypt_password, :sync_encrypt_password],
 		}
 		private_constant :REDIRECT_METHODS
-
-		if PG::Connection.instance_methods.include? :async_encrypt_password
+		if PG::Connection.instance_methods.include? :async_close_prepared
 			REDIRECT_METHODS.merge!({
-				:encrypt_password => [:async_encrypt_password, :sync_encrypt_password],
+				:close_prepared => [:async_close_prepared, :sync_close_prepared],
+				:close_portal => [:async_close_portal, :sync_close_portal],
 			})
 		end
 		PG.make_shareable(REDIRECT_METHODS)
